//
//  ViewController.swift
//  VakcinaDP
//
//  Created by Thomas Hamerník on 03.02.18.
//  Copyright © 2018 Thomas Hamerník. All rights reserved.
//

import UIKit

/// Controller handlig main home view
class HomeViewController: BaseViewController {
    
    // MARK: - UI Objects
    weak var segmentControl: UISegmentedControl!
    weak var tableView: UITableView!
    
    // MARK: - Properties
    private var viewModel: VaccineModeling
    
    /// Computed property. Everytime this value is changed the tableView reload its content
    private var segmentIndex = 0 {
        didSet {
            self.tableView.reloadData()
        }
    }
    
    // MARK: - Life cycle
    init(viewModel: VaccineModeling) {
        self.viewModel = viewModel
        super.init()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
       self.setupTableView()
        
        self.setupActions()
      
        self.getData()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.getData()
    }
    
    
    override func loadView() {
        super.loadView()
        
        let segmentControl = UISegmentedControl(items: ["Pending", "Finished"])
        segmentControl.tintColor = UIColor.init(color: .orangeff)
        segmentControl.selectedSegmentIndex = 0
        self.view.addSubview(segmentControl)
        segmentControl.snp.makeConstraints { (make) in
            make.top.equalTo(view.safeAreaLayoutGuide.snp.top).offset(16)
            make.centerX.equalTo(view)
            make.leading.greaterThanOrEqualTo(20)
            make.trailing.greaterThanOrEqualTo(-20)
        }
        self.segmentControl = segmentControl
        
        let tableView = UITableView()
        tableView.backgroundColor = UIColor.init(color: .backgroundGray)
        tableView.separatorStyle = .none
        self.view.addSubview(tableView)
        tableView.snp.makeConstraints { (make) in
            make.top.equalTo(segmentControl.snp.bottom).offset(10)
            make.leading.trailing.bottom.equalTo(0)
        }
        self.tableView = tableView
        
    }
    
    // MARK: - Setup methods
    /// Setup for tableView
    private func setupTableView() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.separatorStyle = .none
        self.tableView.rowHeight = UITableViewAutomaticDimension
        self.tableView.estimatedRowHeight = 70
    }
    
    /// Setup for actions
   private func setupActions() {
        navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .add, target: self, action: #selector(self.showAddController))
        segmentControl.addTarget(self, action: #selector(self.segmentChanged(sender:)), for: .valueChanged)
    }
    
    // Mark: - Action methods
    /// Call model method for getting all vaccines and then sort them propertly. If it succeed reload table view data.
   private func getData() {
        viewModel.getVaccineData { [weak self](vaccineArray) in
            self?.viewModel.vaccines = vaccineArray
            self?.viewModel.getUserVaccinationAndSortThem(completion: { [weak self] (response) in
                switch response {
                case .success:
                    self?.tableView.reloadData()
                case .failure:
                    print("failure")
                }
            })
        }
    }
    
    
    /// Handle change of segment index. Set the class property to given actual segment index
    @objc func segmentChanged(sender: UISegmentedControl) {
        self.segmentIndex = sender.selectedSegmentIndex
    }
    
    /// Push controller for creating new vaccine record.
   @objc func showAddController() {
    guard let addVc = AppContainer.container.resolve(NewVaccineViewController.self) else { return }
    self.navigationController?.pushViewController(addVc, animated: true)
    }

}

// MARK - Extension for UITableViewDelegate, UITableViewDataSource
extension HomeViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: VaccineTableViewCell = tableView.dequeCellForIndexPath(indexPath: indexPath)
        if self.segmentIndex == 0 {
            cell.setupCell(with: self.viewModel.pending[indexPath.row])
        } else {
            cell.setupCell(with: self.viewModel.finished[indexPath.row])
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if self.segmentIndex == 0 {
            return self.viewModel.pending.count
        } else {
            return self.viewModel.finished.count
        }
    }
    
}

