//
//  SignUpViewController.swift
//  VakcinaDP
//
//  Created by Thomas Hamerník on 03.02.18.
//  Copyright © 2018 Thomas Hamerník. All rights reserved.
//

import UIKit
import FirebaseAuth

/// Controller for signup screen handling
class SignUpViewController: BaseViewController {
    
    // MARK: - UI Objects
    weak var emailTextField: UITextField!
    weak var passwordsTextField: UITextField!
    weak var firstNameTextField: UITextField!
    weak var lastNameTextField: UITextField!
    weak var signUpButton: RoundedButton!
    
    // MARK: - Properties
    private var viewModel: AuthViewModeling

    // MARK: - Life cycle
    override func viewDidLoad() {
        super.viewDidLoad()
        self.signUpButton.addTarget(self, action: #selector(self.didPressSignUp), for: UIControlEvents.touchUpInside)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.navigationController?.navigationBar.isHidden = false
    }

    required init(viewModel: AuthViewModeling) {
        
        self.viewModel = viewModel
        
        super.init()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// Methods which handle setting up the UI using SnapKit framework
    override func loadView() {
        super.loadView()
        
        let firstNameTextField = UITextField()
        firstNameTextField.delegate = self
        firstNameTextField.tag = 0
        firstNameTextField.returnKeyType = .next
        firstNameTextField.placeholder = "First name:"
        firstNameTextField.layer.borderWidth = 1
        firstNameTextField.layer.cornerRadius = 4
        firstNameTextField.layer.borderColor = UIColor.init(color: .orangeff).cgColor
        self.view.addSubview(firstNameTextField)
        firstNameTextField.snp.makeConstraints { (make) in
            make.top.equalTo(view.safeAreaLayoutGuide.snp.top).offset(16)
            make.leading.equalTo(24)
            make.trailing.equalTo(-24)
            make.height.equalTo(54)
        }
        let firstNamePaddingView = UIView(frame: CGRect(x: 0, y: 0, width: 15, height: 54))
        firstNameTextField.leftView = firstNamePaddingView
        firstNameTextField.leftViewMode = .always
        self.firstNameTextField = firstNameTextField
        
        let lastNameTextField = UITextField()
        lastNameTextField.delegate = self
        lastNameTextField.tag = 1
        lastNameTextField.returnKeyType = .next
        lastNameTextField.placeholder = "Last name:"
        lastNameTextField.layer.borderWidth = 1
        lastNameTextField.layer.cornerRadius = 4
        lastNameTextField.layer.borderColor = UIColor.init(color: .orangeff).cgColor
        self.view.addSubview(lastNameTextField)
        lastNameTextField.snp.makeConstraints { (make) in
            make.top.equalTo(firstNameTextField.snp.bottom).offset(20)
            make.leading.equalTo(24)
            make.trailing.equalTo(-24)
            make.height.equalTo(54)
        }
        let lastNamePaddingView = UIView(frame: CGRect(x: 0, y: 0, width: 15, height: 54))
        lastNameTextField.leftView = lastNamePaddingView
        lastNameTextField.leftViewMode = .always
        self.lastNameTextField = lastNameTextField
        
        
        let emailTextField = UITextField()
        emailTextField.delegate = self
        emailTextField.tag = 2
        emailTextField.returnKeyType = .next
        emailTextField.placeholder = "Email address:"
        emailTextField.layer.borderWidth = 1
        emailTextField.layer.cornerRadius = 4
        emailTextField.layer.borderColor = UIColor.init(color: .orangeff).cgColor
        self.view.addSubview(emailTextField)
        emailTextField.snp.makeConstraints { (make) in
            make.top.equalTo(lastNameTextField.snp.bottom).offset(20)
            make.leading.equalTo(24)
            make.trailing.equalTo(-24)
            make.height.equalTo(54)
        }
        let emailPaddingView = UIView(frame: CGRect(x: 0, y: 0, width: 15, height: 54))
        emailTextField.leftView = emailPaddingView
        emailTextField.leftViewMode = .always
        self.emailTextField = emailTextField
        
        let passwordTextField = UITextField()
        passwordTextField.delegate = self
        passwordTextField.tag = 3
        passwordTextField.returnKeyType = .send
        passwordTextField.layer.borderWidth = 1
        passwordTextField.layer.borderColor = UIColor.init(color: .orangeff).cgColor
        passwordTextField.layer.cornerRadius = 4
        passwordTextField.isSecureTextEntry = true
        passwordTextField.placeholder = "Password:"
        self.view.addSubview(passwordTextField)
        passwordTextField.snp.makeConstraints { (make) in
            make.top.equalTo(emailTextField.snp.bottom).offset(20)
            make.leading.equalTo(24)
            make.trailing.equalTo(-24)
            make.height.equalTo(54)
        }
        let passwordPaddingView = UIView(frame: CGRect(x: 0, y: 0, width: 15, height: 54))
        passwordTextField.leftView = passwordPaddingView
        passwordTextField.leftViewMode = .always
        self.passwordsTextField = passwordTextField
        
        let signUpButtonAttributedText = NSAttributedString(string: "Sign Up", attributes: [NSAttributedStringKey.font: UIFont.systemFont(ofSize: 15, weight: UIFont.Weight.bold), NSAttributedStringKey.foregroundColor: UIColor.white])
        
        let signUpButton = RoundedButton()
        signUpButton.backgroundColor = UIColor.init(color: .orangeff)
        signUpButton.setAttributedTitle(signUpButtonAttributedText, for: UIControlState.normal)
        self.view.addSubview(signUpButton)
        signUpButton.snp.makeConstraints { (make) in
            make.top.equalTo(passwordTextField.snp.bottom).offset(40)
            make.leading.equalTo(24)
            make.trailing.equalTo(-24)
            make.height.equalTo(54)
        }
        self.signUpButton = signUpButton
}
    // MARK: - Action methods
    /// Perform sign up acction and change root controller to tabbar controller if succeed
    @objc private func didPressSignUp() {
        guard let email = self.emailTextField.text else { return }
        guard let password = self.passwordsTextField.text else { return }
        guard let firstName = self.firstNameTextField.text else { return }
        guard let lastName = self.lastNameTextField.text else { return }
        
        self.signUpButton.startLoading()
        self.viewModel.signUp(with: email, password: password, firstName: firstName, lastName: lastName) { (response) in
            switch response {
            case .success:
                self.signUpButton.stopLoading()
                let tabBarVc = AppContainer.container ~ TabBarViewController.self
                UIApplication.shared.keyWindow?.rootViewController = tabBarVc
            case .failure:
                self.signUpButton.stopLoading()
                self.presentCommonErrorAlert(with: "Something went wrong", message: "Please repeat or make sure that this account doesn't exist already")
            }
        }
    }
}

extension SignUpViewController: UITextFieldDelegate {
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        let textTag = textField.tag + 1
        let nextResponder = textField.superview?.viewWithTag(textTag)
        if(nextResponder != nil)
        {
            nextResponder?.becomeFirstResponder()
        }
        else {
            self.view.endEditing(true)
            self.didPressSignUp()
        }
        return true
    }
}
