//
//  NewVaccineViewController.swift
//  VakcinaDP
//
//  Created by Thomas Hamerník on 03.02.18.
//  Copyright © 2018 Thomas Hamerník. All rights reserved.
//

import UIKit
import EventKit

/// Controller for new vaccine record
class NewVaccineViewController: BaseViewController {

    // MARK: -  UI Objects
    weak var tableView: UITableView!
    
    // MARK: - Properties
    private let viewModel: VaccineModeling
    
    private var usersVaccinations = [VaccineRecord]()
    private var selectedIndex: Int?
    
    // MARK: - Life cycle
    init(viewModel: VaccineModeling) {
        self.viewModel = viewModel
        super.init()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.viewModel.getUserData { (user) in
            self.usersVaccinations = user.vaccineArray ?? []
        }
        
        self.setupTableView()
    
    }
    
    override func loadView() {
        super.loadView()
        
        let tableView = UITableView()
        tableView.backgroundColor = UIColor.init(color: .backgroundGray)
        view.addSubview(tableView)
        tableView.snp.makeConstraints { (make) in
            make.top.equalTo(view.safeAreaLayoutGuide.snp.top).offset(16)
            make.leading.trailing.bottom.equalTo(0)
        }
        self.tableView = tableView
    }
    
    
    // MARK: - Setup methods
    /// Table view setup method
    private func setupTableView() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.separatorStyle = .none
        self.tableView.rowHeight = UITableViewAutomaticDimension
        self.tableView.estimatedRowHeight = 70
    }
    
    private func addEventToCalendar(title: String, description: String?, startDate: Date, endDate: Date, completion: ((_ success: Bool, _ error: NSError?) -> Void)? = nil) {
        let eventStore = EKEventStore()
        
        eventStore.requestAccess(to: .event, completion: { (granted, error) in
            if (granted) && (error == nil) {
                let event = EKEvent(eventStore: eventStore)
                event.title = title
                event.isAllDay = true
                event.startDate = startDate
                event.endDate = endDate
                event.notes = description
                event.calendar = eventStore.defaultCalendarForNewEvents
                do {
                    try eventStore.save(event, span: .thisEvent)
                } catch let e as NSError {
                    completion?(false, e)
                    return
                }
                completion?(true, nil)
            } else {
                completion?(false, error as NSError?)
            }
        })
    }

}

// MARK: - Extension for UITableViewDataSource, UITableViewDelegate
extension NewVaccineViewController: UITableViewDataSource, UITableViewDelegate {

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: NewVaccineTableViewCell = tableView.dequeCellForIndexPath(indexPath: indexPath)
        cell.setupLabel(with: self.viewModel.vaccines[indexPath.row].name)
        return cell
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return viewModel.vaccines.count
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        guard let pickerVc = AppContainer.container.resolve(DatePickerViewController.self) else { return }
        pickerVc.modalPresentationStyle = .overCurrentContext
        pickerVc.delegate = self
        self.selectedIndex = indexPath.row
       DispatchQueue.main.async { [weak self] in
            self?.tabBarController?.present(pickerVc, animated: true, completion: nil)
        }
    }
}

// MARK: - Extension for DatePickerViewControllerDelegate
extension NewVaccineViewController: DatePickerViewControllerDelegate {
    func didConfirmPick(with date: Date) {
        guard let index = self.selectedIndex else { return }
        self.usersVaccinations.append(VaccineRecord(vaccineId: self.viewModel.vaccines[index].id , vaccinationDate: date))
        viewModel.writeVaccineToDatabase(vaccines: self.usersVaccinations) { (response) in
            switch response {
            case .success:
                self.addEventToCalendar(title:"Vaccination for: \(self.viewModel.vaccines[index].name)", description: nil, startDate: date, endDate: date, completion: { (bool, error) in
                    if error == nil {
                         self.presentCommonErrorAlert(with: "Vaccination record added!", message: "You going to be healthy!")
                        
                    } else {
                        self.presentCommonErrorAlert(with: "Something went wrong", message: "Please try again")
                    }
                })
            case .failure:
                 self.presentCommonErrorAlert(with: "Something went wrong", message: "Please try again")
            }
        }
    }
}
