//
//  AuthService.swift
//  VakcinaDP
//
//  Created by Thomas Hamerník on 04.02.18.
//  Copyright © 2018 Thomas Hamerník. All rights reserved.
//

import Foundation
import FirebaseAuth
import FirebaseDatabase

/// Service for handling auth operations
protocol AuthServicing {
    /// Create new user for given input and pass data to databaseService for making a record.
    ///
    /// - Parameters:
    ///   - email: Users email
    ///   - password: Users password
    ///   - firstName: Users first name
    ///   - lastName: Users last name
    ///   - completion: Common completion block for Success or Failure
    /// - Returns: Void
    func signUp(with email: String, password: String, firstName: String, lastName: String, completion: @escaping (_ response: CommonResponse) -> Void)
    
    /// Authenticate already existing user against firebase and download users data through databaesService. Users data are then saved in keychain.
    ///
    /// - Parameters:
    ///   - email: Users email
    ///   - password: Users password
    ///   - completion: Common completion block for Success or Failure
    /// - Returns: Void
    func login(with email: String, password: String, completion: @escaping (_ response: CommonResponse) -> Void)
    
    /// Sign out currently logged user
    ///
    /// - Parameter completion: Common completion block for Success or Failure
    func signOut(with completion: @escaping (_ response: CommonResponse) -> Void)
}

class AuthService: AuthServicing {
    
    private let databaseService: DatabaseFetchServicing
    
    required init(databaseService: DatabaseFetchServicing) {
        self.databaseService = databaseService
    }
    
    func signUp(with email: String, password: String, firstName: String, lastName: String, completion: @escaping (_ response: CommonResponse) -> Void) {
        
        Auth.auth().createUser(withEmail: email, password: password) { [weak self] (user, error) in
            if error == nil {
                if let id = user?.uid {
                    let userToWrite = User(id: id, firstName: firstName, lastName: lastName, email: email, vaccineArray: [] )
                    userToWrite.store()
                    self?.databaseService.writeUserToDatabase(user: userToWrite)
                    completion(.success)
                } else {
                    completion(.failure)
                }
            } else {
                completion(.failure)
            }
            
        }
    }
    
    func login(with email:String, password: String, completion: @escaping (_ response: CommonResponse) -> Void) {
        Auth.auth().signIn(withEmail: email, password: password) { [weak self] (user, error) in
            if error == nil {
                if let id = user?.uid {
                    self?.databaseService.getUserDataFromDatabase(for: id, completion: { (user) in
                        user.store()
                        completion(.success)
                    })
                } else {
                    completion(.failure)
                }
            } else {
                completion(.failure)
            }
        }
    }
    
    func signOut(with completion: @escaping (_ response: CommonResponse) -> Void) {
        do {
            try Auth.auth().signOut()
            StorageHelper.deleteUserInformations()
            completion(.success)
        } catch {
            completion(.failure)
        }
    }
    
}
