//
//  VaccineService.swift
//  VakcinaDP
//
//  Created by Thomas Hamerník on 06.02.18.
//  Copyright © 2018 Thomas Hamerník. All rights reserved.
//

import Foundation

/// Model service for wrapping other lower services and handling addiotional behaviour needed in controllers
protocol VaccineModeling {
    /// Array of all vaccines
    var vaccines: [Vaccine] { get set }
    
    /// Array of finished vaccines
    var finished: [Vaccine] { get set }
    
    /// Array of pending vaccines
    var pending: [Vaccine] { get set }
    
    /// Get user data and set finished and pending vaccinations
    ///
    /// - Parameter completion: Common completion block for Success or Failure
    /// - Returns: Void
    func getUserVaccinationAndSortThem(completion: @escaping (_ response: CommonResponse) -> Void)
    
    /// Wrap database service to get users data
    ///
    /// - Parameter completion: Completion block return downloaded User
    /// - Returns: Void
    func getUserData(completion: @escaping (_ response: User) -> Void)
    
    /// Wrap database service method to get all vaccines data from database
    ///
    /// - Parameter completion: Completion block return all downloaded Vaccines
    /// - Returns: Void
    func getVaccineData(completion: @escaping (_ response: [Vaccine]) -> Void)
    
    /// Wrape database service methods to write newly appointed vaccine do database for selected user.
    ///
    /// - Parameters:
    ///   - vaccines: Array of VaccineRecords whitch containt selected date and vaccine ID
    ///   - completion: Common completion block for Success or Failure
    func writeVaccineToDatabase(vaccines: [VaccineRecord]?, completion: @escaping (_ response: CommonResponse) -> Void )
}

class VaccineModel: VaccineModeling {
    
    
    var vaccines: [Vaccine] = []
    var finished: [Vaccine] = []
    var pending: [Vaccine] = []
    
    private let databaseService: DatabaseFetchServicing
    
    required init(databaseService: DatabaseFetchServicing) {
        self.databaseService = databaseService
        
    }
    
    func getUserData(completion: @escaping (_ response: User) -> Void) {
        guard let userId = StorageHelper.user?.id else { return }
        self.databaseService.getUserDataFromDatabase(for: userId, completion: completion)
    }
    
    func getVaccineData(completion: @escaping (_ response: [Vaccine]) -> Void) {
        databaseService.getVaccinesDataFromDatabase { (vaccineArray) in
            completion(vaccineArray)
        }
    }
    func writeVaccineToDatabase(vaccines: [VaccineRecord]?, completion: @escaping (CommonResponse) -> Void) {
        guard let user = StorageHelper.user else {
            completion(.failure)
            return
        }
        databaseService.writeUserToDatabase(user: User(id: user.id, firstName: user.firstName, lastName: user.lastName, email: user.email, vaccineArray: vaccines))
        completion(.success)
    }
    
      /// Sort vaccines into different arrays for proper UI presentation.
      ///
      /// - Parameters:
      ///   - user: Current user
      ///   - vaccines: All vaccines avaliable
      /// - Returns: Tuple of finished and pending vaccines arrays.
      func vaccineSort(with user: User, vaccines: [Vaccine]) -> (finished: [Vaccine], pending: [Vaccine]) {
        var finished = [Vaccine]()
        var pending = [Vaccine]()
        
        guard let vaccineArray = user.vaccineArray else { return (finished,pending) }
        
        let finishedRecords = vaccineArray.filter({ (record) -> Bool in
            guard let date = record.vaccinationDate else { return false }
            return date < Date()
        })
        
        let pendingRecords = vaccineArray.filter({ (record) -> Bool in
            guard let date = record.vaccinationDate else { return false }
            return date >= Date()
        })
        
        for record in finishedRecords {
            for var vaccine in vaccines {
                if vaccine.id == record.vaccineId {
                    vaccine.vaccinationDate = record.vaccinationDate
                    finished.append(vaccine)
                }
            }
        }
        
        for record in pendingRecords {
            for var vaccine in vaccines {
                if vaccine.id == record.vaccineId {
                    vaccine.vaccinationDate = record.vaccinationDate
                    pending.append(vaccine)
                }
            }
        }
        
        
        return (finished, pending)
    }
    
    
    func getUserVaccinationAndSortThem(completion: @escaping (CommonResponse) -> Void) {
        guard let userId = StorageHelper.user?.id else {
            completion(.failure)
            return
        }
        databaseService.getUserDataFromDatabase(for: userId) { (user) in
            
           let filterTuple = self.vaccineSort(with: user, vaccines: self.vaccines)
            self.finished = filterTuple.finished
            self.pending = filterTuple.pending
            
            completion(.success)
        }
    }
}
